@@ -2,9 +2,11 @@ import numpy as np
 from numpy.linalg import LinAlgError, inv, pinv
 from numpy import dot
 import pandas as pd
+from numpy.random import beta
 
-from lifelines.plotting import plot_dataframes, plot_regressions
-from lifelines.utils import survival_table_from_events, basis, inv_normal_cdf, quadrature, epanechnikov_kernel
+from lifelines.plotting import plot_estimate, plot_regressions
+from lifelines.utils import survival_table_from_events, inv_normal_cdf, quadrature, \
+                            epanechnikov_kernel, StatError, coalesce
 
 
 class NelsonAalenFitter(object):
@@ -30,46 +32,34 @@ class NelsonAalenFitter(object):
             self._variance_f = self._variance_f_discrete
             self._additive_f = self._additive_f_discrete
 
-    def fit(self, event_times, censorship=None, timeline=None, label='NA-estimate', alpha=None, insert_0=True):
+    def fit(self, durations, event_observed=None, timeline=None, entry=None, label='NA-estimate', alpha=None):
         """
         Parameters:
-          event_times: an array, or pd.Series, of length n of times that the death event occured at
+          duration: an array, or pd.Series, of length n -- duration subject was observed for
           timeline: return the best estimate at the values in timelines (postively increasing)
-          censorship: an array, or pd.Series, of length n -- True if the the death was observed, False if the event
-             was lost (right-censored). Defaults all True if censorship==None
+          event_observed: an array, or pd.Series, of length n -- True if the the death was observed, False if the event
+             was lost (right-censored). Defaults all True if event_observed==None
+          entry: an array, or pd.Series, of length n -- relative time when a subject entered the study. This is 
+             useful for left-truncated observations, i.e the birth event was not observed. 
+             If None, defaults to all 0 (all birth events observed.)
           label: a string to name the column of the estimate.
           alpha: the alpha value in the confidence intervals. Overrides the initializing
              alpha for this call to fit only.
-          insert_0: add a leading 0 (if not present) in the timeline.
 
         Returns:
-          self, with new properties like 'survival_function_'.
+          self, with new properties like 'cumulative_hazard_'.
 
         """
 
-        if censorship is None:
-            self.censorship = np.ones(len(event_times), dtype=bool)
-        else:
-            self.censorship = np.array(censorship).copy().astype(bool)
+        v = preprocess_inputs(durations, event_observed, timeline, entry)
+        self.durations, self.event_observed, self.timeline, self.entry, self.event_table = v
 
-        self.event_times = survival_table_from_events(event_times, self.censorship)
-
-        if alpha is None:
-            alpha = self.alpha
-
-        if timeline is None:
-            self.timeline = self.event_times.index.values.copy().astype(float)
-            if insert_0 and self.timeline[0] > 0:
-                self.timeline = np.insert(self.timeline, 0, 0.)
-        else:
-            self.timeline = np.array(timeline).copy().astype(float)
-
-        cumulative_hazard_, cumulative_sq_ = _additive_estimate(self.event_times, self.timeline,
+        cumulative_hazard_, cumulative_sq_ = _additive_estimate(self.event_table, self.timeline,
                                                                 self._additive_f, self._variance_f)
 
         # esimates
         self.cumulative_hazard_ = pd.DataFrame(cumulative_hazard_, columns=[label])
-        self.confidence_interval_ = self._bounds(cumulative_sq_[:, None], alpha)
+        self.confidence_interval_ = self._bounds(cumulative_sq_[:, None], alpha if alpha else self.alpha)
         self._cumulative_sq = cumulative_sq_
 
         # estimation functions
@@ -78,9 +68,9 @@ class NelsonAalenFitter(object):
         self.divide = _divide(self, "cumulative_hazard_")
 
         # plotting
-        self.plot = plot_dataframes(self, "cumulative_hazard_")
+        self.plot = plot_estimate(self, "cumulative_hazard_")
         self.plot_cumulative_hazard = self.plot
-        self.plot_hazard = plot_dataframes(self, 'hazard_')
+        self.plot_hazard = plot_estimate(self, 'hazard_')
 
         return self
 
@@ -148,8 +138,8 @@ class NelsonAalenFitter(object):
 
     def __repr__(self):
         try:
-            s = """<lifelines.NelsonAalenFitter: fitted with %d observations, %d censored.>""" % (
-                self.censorship.shape[0], (~self.censorship).sum())
+            s = """<lifelines.NelsonAalenFitter: fitted with %d observations, %d censored>""" % (
+                self.event_observed.shape[0], (1-self.event_observed).sum())
         except AttributeError as e:
             s = """<lifelines.NelsonAalenFitter>"""
         return s
@@ -157,49 +147,57 @@ class NelsonAalenFitter(object):
 
 class KaplanMeierFitter(object):
 
+    """
+    Class for fitting the Kaplan-Meier estimate for the survival function.
+
+    KaplanMeierFitter( alpha=0.95)
+
+    alpha: The alpha value associated with the confidence intervals.
+
+    """
+
+
     def __init__(self, alpha=0.95):
         self.alpha = alpha
 
-    def fit(self, event_times, censorship=None, timeline=None, label='KM-estimate', alpha=None, insert_0=True):
+    def fit(self, durations, event_observed=None, timeline=None, entry=None, label='KM-estimate', alpha=None):
         """
         Parameters:
-           event_times: an array, or pd.Series, of length n of times that the death event occured at
-           timeline: return the best estimate at the values in timelines (postively increasing)
-           censorship: an array, or pd.Series, of length n -- True if the the death was observed, False if the event
-              was lost (right-censored). Defaults all True if censorship==None
-           label: a string to name the column of the estimate.
-           alpha: the alpha value in the confidence intervals. Overrides the initializing
-              alpha for this call to fit only.
-           insert_0: add a leading 0 (if not present) in the timeline.
+          duration: an array, or pd.Series, of length n -- duration subject was observed for
+          timeline: return the best estimate at the values in timelines (postively increasing)
+          event_observed: an array, or pd.Series, of length n -- True if the the death was observed, False if the event
+             was lost (right-censored). Defaults all True if event_observed==None
+          entry: an array, or pd.Series, of length n -- relative time when a subject entered the study. This is 
+             useful for left-truncated observations, i.e the birth event was not observed. 
+             If None, defaults to all 0 (all birth events observed.)
+          label: a string to name the column of the estimate.
+          alpha: the alpha value in the confidence intervals. Overrides the initializing
+             alpha for this call to fit only.
 
         Returns:
-           self, with new properties like 'survival_function_'.
+          self, with new properties like 'survival_function_'.
 
         """
-        # set to all observed if censorship is none
-        if censorship is None:
-            self.censorship = np.ones(len(event_times), dtype=bool)
-        else:
-            self.censorship = np.array(censorship).copy().astype(bool)
+        v = preprocess_inputs(durations, event_observed, timeline, entry)
+        self.durations, self.event_observed, self.timeline, self.entry, self.event_table = v
 
-        if not alpha:
-            alpha = self.alpha
-
-        self.event_times = survival_table_from_events(event_times, self.censorship)
-
-        if timeline is None:
-            self.timeline = self.event_times.index.values.copy().astype(float)
-            if insert_0 and self.timeline[0] > 0:
-                self.timeline = np.insert(self.timeline, 0, 0.)
-        else:
-            self.timeline = timeline.astype(float)
-
-        log_survival_function, cumulative_sq_ = _additive_estimate(self.event_times, self.timeline,
+        log_survival_function, cumulative_sq_ = _additive_estimate(self.event_table, self.timeline,
                                                                    self._additive_f, self._additive_var)
 
+        if entry is not None:
+            #a serious problem with KM is that when the sample size is small and there are too few early 
+            # truncation times, it may happen that is the number of patients at risk and the number of deaths is the same.
+            # we adjust for this using the Breslow-Fleming-Harrington estimator 
+            n = self.event_table.shape[0]
+            net_population = (self.event_table['entrance'] - self.event_table['removed']).cumsum()
+            if net_population.iloc[:int(n/2)].min() == 0:
+                ix = net_population.iloc[:int(n/2)].argmin()
+                raise StatError("""There are too few early truncation times and too many events. S(t)==0 for all t>%.1f. Recommend BFH estimator."""%ix)
+
+
         # estimation
         self.survival_function_ = pd.DataFrame(np.exp(log_survival_function), columns=[label])
-        self.confidence_interval_ = self._bounds(cumulative_sq_[:, None], alpha)
+        self.confidence_interval_ = self._bounds(cumulative_sq_[:, None], alpha if alpha else self.alpha)
         self.median_ = median_survival_times(self.survival_function_)
 
         # estimation methods
@@ -208,7 +206,7 @@ class KaplanMeierFitter(object):
         self.divide = _divide(self, "survival_function_")
 
         # plotting functions
-        self.plot = plot_dataframes(self, "survival_function_")
+        self.plot = plot_estimate(self, "survival_function_")
         self.plot_survival_function = self.plot
         return self
 
@@ -233,14 +231,130 @@ class KaplanMeierFitter(object):
     def __repr__(self):
         try:
             s = """<lifelines.KaplanMeierFitter: fitted with %d observations, %d censored>""" % (
-                self.censorship.shape[0], (~self.censorship).sum())
+                self.event_observed.shape[0], (1-self.event_observed).sum())
         except AttributeError as e:
             s = """<lifelines.KaplanMeierFitter>"""
         return s
 
 
-class AalenAdditiveFitter(object):
+class BreslowFlemingHarringtonFitter(object):
 
+    """
+    Class for fitting the Breslow-Fleming-Harrington estimate for the survival function. This estimator 
+    is a biased estimator of the survival function but is more stable when the popualtion is small and 
+    there are too few early truncation times, it may happen that is the number of patients at risk and 
+    the number of deaths is the same.
+
+    Mathematically, the NAF estimator is the negative logarithm of the BFH estimator.
+
+    BreslowFlemingHarringtonFitter(alpha=0.95)
+
+    alpha: The alpha value associated with the confidence intervals.
+
+    """
+
+    def __init__(self, alpha=0.95):
+        self.alpha = alpha
+
+    def fit(self, durations, event_observed=None, timeline=None, entry=None, label='BFH-estimate', alpha=None):
+        """
+        Parameters:
+          duration: an array, or pd.Series, of length n -- duration subject was observed for
+          timeline: return the best estimate at the values in timelines (postively increasing)
+          event_observed: an array, or pd.Series, of length n -- True if the the death was observed, False if the event
+             was lost (right-censored). Defaults all True if event_observed==None
+          entry: an array, or pd.Series, of length n -- relative time when a subject entered the study. This is 
+             useful for left-truncated observations, i.e the birth event was not observed. 
+             If None, defaults to all 0 (all birth events observed.)
+          label: a string to name the column of the estimate.
+          alpha: the alpha value in the confidence intervals. Overrides the initializing
+             alpha for this call to fit only.
+
+        Returns:
+          self, with new properties like 'survival_function_'.
+
+        """
+        naf = NelsonAalenFitter(self.alpha)
+        naf.fit(durations, event_observed=event_observed, timeline=timeline, label=label, entry=entry)
+        self.durations, self.event_observed, self.timeline, self.entry, self.event_table = \
+                naf.durations, naf.event_observed, naf.timeline, naf.entry, naf.event_table
+
+        # estimation
+        self.survival_function_ = np.exp(-naf.cumulative_hazard_)
+        self.confidence_interval_ = np.exp(-naf.confidence_interval_)
+        self.median_ = median_survival_times(self.survival_function_)
+
+        # estimation methods
+        self.predict = _predict(self, "survival_function_", label)
+        self.subtract = _subtract(self, "survival_function_")
+        self.divide = _divide(self, "survival_function_")
+
+        # plotting functions
+        self.plot = plot_estimate(self, "survival_function_")
+        self.plot_survival_function = self.plot
+        return self
+
+
+class BayesianFitter(object):
+    """
+    If you have small data, and KM feels too uncertain, you can use the BayesianFitter to 
+    generate sample survival functions. The algorithm is:
+
+    S_i(T) = \Prod_{t=0}^T (1 - p_t)
+
+    where p_t ~ Beta( 0.01 + d_t, 0.01 + n_t - d_t), d_t is the number of deaths and n_t is the size of the 
+    population at risk at time t. The prior is a Beta(0.01, 0.01) for each time point (high values led to a 
+    high bias).
+
+    Parameters:
+        samples: the number of sample survival functions to return.
+
+    """
+
+    def __init__(self, samples = 100):
+        self.beta = beta
+        self.samples = samples
+
+    def fit(self, durations, censorship=None, timeline=None, entry=None):
+        """
+        Parameters:
+          duration: an array, or pd.Series, of length n -- duration subject was observed for
+          timeline: return the best estimate at the values in timelines (postively increasing)
+          censorship: an array, or pd.Series, of length n -- True if the the death was observed, False if the event
+             was lost (right-censored). Defaults all True if censorship==None
+          entry: an array, or pd.Series, of length n -- relative time when a subject entered the study. This is 
+             useful for left-truncated observations, i.e the birth event was not observed. 
+             If None, defaults to all 0 (all birth events observed.)
+
+        Returns:
+          self, with new properties like 'sample_survival_functions_'.
+        """
+        v = preprocess_inputs(durations, censorship, timeline, entry)
+        self.durations, self.censorship, self.timeline, self.entry, self.event_table = v
+
+        self.sample_survival_functions_ = self.generate_sample_path(self.samples)
+
+        return self
+
+    def plot(self, **kwargs):
+        kwargs['alpha'] = coalesce(kwargs.get('alpha'), 15./self.samples)
+        kwargs['legend'] = False
+        kwargs['c'] = coalesce( kwargs.get('c'), kwargs.get('color'), '#348ABD')
+        ax = self.sample_survival_functions_.plot(**kwargs)
+        return ax
+
+    def generate_sample_path(self, n=1):
+
+        deaths = self.event_table['observed']
+        population = self.event_table['entrance'].cumsum() - self.event_table['removed'].cumsum().shift(1).fillna(0)
+        d = deaths.shape[0]
+        samples = 1. - beta( 0.01 + deaths, 0.01 + population - deaths, size=(n, d))
+        sample_paths = pd.DataFrame( np.exp(np.log(samples).cumsum(1)).T, index = self.timeline )
+        return sample_paths
+
+
+
+class AalenAdditiveFitter(object):
     """
     This class fits the regression model:
 
@@ -262,112 +376,125 @@ class AalenAdditiveFitter(object):
         self.fit_intercept = fit_intercept
         self.alpha = alpha
         self.penalizer = penalizer
-        assert penalizer >= 0, "penalizer keyword must be >= 0."
+        assert penalizer >= 0, "penalizer must be >= 0."
 
-    def fit(self, event_times, X, timeline=None, censorship=None, columns=None):
-        """currently X is a static (n,d) array
-
-        event_times: (n,1) array of event times
-        X: (n,d) the design matrix, either a numpy matrix or DataFrame.
-        timeline: (t,1) timepoints in ascending order
-        censorship: (n,1) boolean array of censorships: True if observed, False if right-censored.
-                    By default, assuming all are observed.
-
-        Fits: self.cumulative_hazards_: a (t,d+1) dataframe of cumulative hazard coefficients
-              self.hazards_: a (t,d+1) dataframe of hazard coefficients
 
+    def fit(self, dataframe, duration_col="T", event_col="E", timeline=None, id_col=None):
         """
-        # deal with the covariate matrix. Check if it is a dataframe or numpy array
-        n, d = X.shape
-        if type(X) == pd.core.frame.DataFrame:
-            X_ = X.values.copy()
-            if columns is None:
-                columns = X.columns
-        else:
-            X_ = X.copy()
+        Perform inference on the coefficients of the Aalen additive model. 
 
-        # append a columns of ones for the baseline hazard
-        ix = event_times.argsort(0)[:, 0].copy()
-        X_ = X_[ix,:].copy() if not self.fit_intercept else np.c_[ X_[ix,:].copy(), np.ones((n, 1)) ]
-        sorted_event_times = event_times[ix, 0].copy()
+        Parameters:
+            dataframe: a pandas dataframe, with covariates and a duration_col and a event_col.
 
-        # set the column's names of the dataframe.
-        if columns is None:
-            columns = range(d)
-        else:
-            columns = [c for c in columns]
+                static covariates:
+                    one row per individual. duration_col refers to how long the individual was 
+                      observed for. event_col is a boolean: 1 if individual 'died', 0 else. id_col 
+                      should be left as None.
+
+                time-varying covariates:
+                    For time-varying covariates, an id_col is required to keep track of individuals'
+                    changing covariates. individual should have a unique id. duration_col refers to how 
+                    long the individual has been  observed to up to that point. event_col refers to if 
+                    the event (death) occured in that  period. Censored individuals will not have a 1. 
+                    For example:
+
+                        +----+---+---+------+------+
+                        | id | T | E | var1 | var2 |
+                        +----+---+---+------+------+
+                        |  1 | 1 | 0 |    0 |    1 |
+                        |  1 | 2 | 0 |    0 |    1 |
+                        |  1 | 3 | 0 |    4 |    3 |
+                        |  1 | 4 | 1 |    8 |    4 |
+                        |  2 | 1 | 0 |    1 |    1 |
+                        |  2 | 2 | 0 |    1 |    2 |
+                        |  2 | 3 | 0 |    1 |    2 |
+                        +----+---+---+------+------+
+
+            duration_col: specify what the duration column is called in the dataframe 
+            event_col: specify what the event occurred column is called in the dataframe 
+            timeline: reformat the estimates index to a new timeline.
+            id_col: (only for time-varying covariates) name of the id column in the dataframe
+
+
+        Returns:
+          self, with new methods like plot, smoothed_hazards_ and properties like cumulative_hazards_
+        """
+
+        df = dataframe.copy()
+        if id_col is None:
+            #only for time-indp. covariates
+            df['id'] = np.arange(df.shape[0])
+            id_col = 'id'
 
         if self.fit_intercept:
-            columns += ['baseline']
+            df['baseline'] = 1.
 
-        # set the censorship events. 1 if the death was observed.
-        if censorship is None:
-            observed = np.ones(n, dtype=bool)
-        else:
-            observed = censorship[ix].reshape(n)
+        df = df.set_index([id_col, duration_col])
+  
+        C_panel = df[[event_col]].to_panel().transpose(1,2,0)
+        C = C_panel.minor_xs(event_col).sum().astype(bool)
+        T = (C_panel.minor_xs(event_col).notnull()).cumsum().idxmax()
 
-        # set the timeline -- this is used as DataFrame index in the results
-        if timeline is None:
-            timeline = sorted_event_times.copy()
+        del df[event_col]
+        n,d = df.shape
 
-        timeline = np.unique(timeline.astype(float))
-        if timeline[0] > 0:
-            timeline = np.insert(timeline, 0, 0.)
+        from_tuples = pd.MultiIndex.from_tuples
+        wp = df.to_panel().transpose(1,2,0).bfill().fillna(0) #bfill will cause problems later, plus it is slow.
 
-        unique_times = np.unique(timeline)
-        zeros = np.zeros((timeline.shape[0], d + self.fit_intercept))
-        self.cumulative_hazards_ = pd.DataFrame(zeros.copy(), index=unique_times, columns=columns)
-        self.hazards_ = pd.DataFrame(
-            np.zeros((event_times.shape[0], d + self.fit_intercept)), index=event_times[:, 0], columns=columns)
-        self._variance = pd.DataFrame(zeros.copy(), index=unique_times, columns=columns)
+        non_censorsed_times = T[C].iteritems()
+        hazards_ = pd.DataFrame( np.zeros((len(non_censorsed_times),d)), 
+                        columns = df.columns, index = from_tuples(non_censorsed_times))
 
-        # create the penalizer matrix for L2 regression
-        penalizer = self.penalizer * np.eye(d + self.fit_intercept)
+        variance_  = pd.DataFrame( np.zeros((len(non_censorsed_times),d)), 
+                        columns = df.columns, index = from_tuples(non_censorsed_times))
+        
+        ids = wp.items
+        penalizer = self.penalizer*np.eye(d)
+        
+        for i,(id, time) in enumerate(non_censorsed_times): 
 
-        t_0 = sorted_event_times[0]
-        cum_v = np.zeros((d + self.fit_intercept, 1))
-        v = cum_v.copy()
-        for i, time in enumerate(sorted_event_times):
-            relevant_times = (t_0 < timeline) * (timeline <= time)
-            if observed[i] == 0:
-                X_[i,:] = 0
+            relevant_individuals = (ids==id)
+            assert relevant_individuals.sum() == 1.
+
+            X = wp.major_xs(time).values.T
+
+            #perform linear regression step.
             try:
-                V = dot(inv(dot(X_.T, X_) + penalizer), X_.T)
+                V = np.dot(pinv(np.dot(X.T, X) + penalizer), X.T)
             except LinAlgError:
-                # if penalizer > 0, this should not occur. But sometimes it does...
-                V = dot(pinv(dot(X_.T, X_) + penalizer), X_.T)
+                print "Linear regression error. Try increasing the penalizer."
+            v = np.dot(V, 1.0*relevant_individuals )
 
-            v = dot(V, basis(n, i))
-            cum_v = cum_v + v
-            self.cumulative_hazards_.ix[relevant_times] = self.cumulative_hazards_.ix[relevant_times].values + cum_v.T
-            self.hazards_.iloc[i] = self.hazards_.iloc[i].values + v.T
-            self._variance.ix[relevant_times] = self._variance.ix[relevant_times].values + dot( V[:, i][:, None], V[:, i][None,:] ).diagonal()
-            t_0 = time
-            X_[i,:] = 0
+            hazards_.ix[id, time]  = v.T
+            variance_.ix[id, time] = np.dot( V[:, relevant_individuals], V[:, relevant_individuals].T ).diagonal()
 
-        # clean up last iteration
-        relevant_times = (timeline > time)
-        self.hazards_.iloc[i] = v.T
-        try:
-            self.cumulative_hazards_.ix[relevant_times] = cum_v.T
-            self._variance.ix[relevant_times] = dot( V[:, i][:, None], V[:, i][None,:] ).diagonal()
-        except:
-            pass
-        self.timeline = timeline
-        self.X = X
-        self.censorship = censorship
-        self.event_times = event_times
+        #not sure this is the correct thing to do.
+        self.hazards_ = hazards_.groupby(level=1).sum()
+        self.cumulative_hazards_= self.hazards_.cumsum()
+        self.variance_ = variance_.groupby(level=1).sum()
+
+        if timeline is not None:
+            self.hazards_ = self.hazards_.reindex(timeline, method='ffill')
+            self.cumulative_hazards_ = self.cumulative_hazards_.reindex(timeline, method='ffill')
+            self.variance_= self.variance_.reindex(timeline, method='ffill')
+            self.timeline = timeline
+        else:
+            self.timeline = self.hazards_.index.values.astype(float)
+
+        self.data = wp
+
+        self.durations = T
+        self.event_occured = C
         self._compute_confidence_intervals()
         self.plot = plot_regressions(self)
         return self
 
-    def smoothed_hazards_(self, bandwith=1):
+    def smoothed_hazards_(self, bandwidth=1):
         """
-        Using the epanechnikov kernel to smooth the hazard function, with sigma/bandwith
+        Using the epanechnikov kernel to smooth the hazard function, with sigma/bandwidth
 
         """
-        C = self.censorship.astype(bool)
-        return pd.DataFrame( np.dot(epanechnikov_kernel(self.timeline[:, None], self.timeline[C], bandwith), self.hazards_.values[C,:]),
+        return pd.DataFrame( np.dot(epanechnikov_kernel(self.timeline[:, None], self.timeline, bandwidth), self.hazards_.values),
                              columns=self.hazards_.columns, index=self.timeline)
 
     def _compute_confidence_intervals(self):
@@ -375,20 +502,29 @@ class AalenAdditiveFitter(object):
         n = self.timeline.shape[0]
         d = self.cumulative_hazards_.shape[1]
         index = [['upper'] * n + ['lower'] * n, np.concatenate([self.timeline, self.timeline])]
-        self.confidence_intervals_ = pd.DataFrame(
-            np.zeros((2 * n, d)), index=index, columns=self.cumulative_hazards_.columns)
+
+        self.confidence_intervals_ = pd.DataFrame( np.zeros((2 * n, d)), 
+                                                    index=index, 
+                                                    columns=self.cumulative_hazards_.columns
+                                                  )
+
         self.confidence_intervals_.ix['upper'] = self.cumulative_hazards_.values + \
-            alpha2 * np.sqrt(self._variance.cumsum().values)
+                alpha2 * np.sqrt(self.variance_.cumsum().values)
+
         self.confidence_intervals_.ix['lower'] = self.cumulative_hazards_.values - \
-            alpha2 * np.sqrt(self._variance.cumsum().values)
+                alpha2 * np.sqrt(self.variance_.cumsum().values)
         return
 
-    def predict_cumulative_hazard(self, X, columns=None):
+    def predict_cumulative_hazard(self, X, columns=None, id_col=None):
         """
         X: a (n,d) covariate matrix
 
         Returns the hazard rates for the individuals
         """
+        if id_col is not None:
+            #see https://github.com/CamDavidsonPilon/lifelines/issues/38
+            raise NotImplementedError
+
         n, d = X.shape
         try:
             X_ = X.values.copy()
@@ -476,16 +612,40 @@ def _predict(self, estimate, label):
     predict.__doc__ = doc_string
     return predict
 
+def preprocess_inputs(durations, event_observed, timeline, entry ):
+
+    n = len(durations)
+    durations = np.asarray(durations).reshape((n,))
+
+    # set to all observed if event_observed is none
+    if event_observed is None:
+        event_observed = np.ones(n, dtype=int)
+    else:
+        event_observed = np.asarray(event_observed).reshape((n,)).copy().astype(int)
+
+    if entry is None:
+        entry = np.zeros(n)
+    else:
+        entry = np.asarray(entry).reshape((n,))
+
+    event_table = survival_table_from_events(durations, event_observed, entry)
+
+    if timeline is None:
+        timeline = event_table.index.values
+    else:
+        timeline = np.asarray(timeline)
+
+    return durations, event_observed, timeline.astype(float), entry, event_table
+
 
 def _additive_estimate(events, timeline, _additive_f, _additive_var):
     """
     Called to compute the Kaplan Meier and Nelson-Aalen estimates.
 
     """
-    N = events["removed"].sum()
 
     deaths = events['observed']
-    population = N - events['removed'].cumsum().shift(1).fillna(0)
+    population = events['entrance'].cumsum() - events['removed'].cumsum().shift(1).fillna(0) #slowest line here.
     estimate_ = np.cumsum(_additive_f(population, deaths))
     var_ = np.cumsum(_additive_var(population, deaths))
 
@@ -515,13 +675,14 @@ def qth_survival_times(q, survival_functions):
     sv_b = (1.0 * (survival_functions < q)).cumsum() > 0
     try:
         v = sv_b.idxmax(0)
-        v[sv_b.iloc[-1,:] == 0] = -1
+        v[sv_b.iloc[-1,:] == 0] = np.inf
     except:
         v = sv_b.argmax(0)
         v[sv_b[-1,:] == 0] = np.inf
     return v
 
 
+
 def median_survival_times(survival_functions):
     return qth_survival_times(0.5, survival_functions)
 
