@@ -4,12 +4,13 @@ from __future__ import print_function
 import numpy as np
 from matplotlib import pyplot as plt
 
+from lifelines.utils import coalesce
 
-def plot_lifetimes(lifetimes, censorship=None, birthtimes=None, order=False):
+def plot_lifetimes(lifetimes, event_observed=None, birthtimes=None, order=False):
     """
     Parameters:
       lifetimes: an (n,) numpy array of lifetimes.
-      censorship: an (n,) numpy array of booleans: True if event observed, else False.
+      event_observed: an (n,) numpy array of booleans: True if event observed, else False.
       birthtimes: an (n,) numpy array offsetting the births away from t=0.
 
 
@@ -21,8 +22,8 @@ def plot_lifetimes(lifetimes, censorship=None, birthtimes=None, order=False):
     if N > 100:
         print("warning: you may want to subsample to less than 100 individuals.")
 
-    if censorship is None:
-        censorship = np.ones(N, dtype=bool)
+    if event_observed is None:
+        event_observed = np.ones(N, dtype=bool)
 
     if birthtimes is None:
         birthtimes = np.zeros(N)
@@ -31,13 +32,13 @@ def plot_lifetimes(lifetimes, censorship=None, birthtimes=None, order=False):
         """order by length of lifetimes; probably not very informative."""
         ix = np.argsort(lifetimes, 0)
         lifetimes = lifetimes[ix, 0]
-        censorship = censorship[ix, 0]
+        event_observed = event_observed[ix, 0]
         birthtimes = birthtimes[ix]
 
     for i in range(N):
-        c = "#A60628" if censorship[i] else "#348ABD"
+        c = "#A60628" if event_observed[i] else "#348ABD"
         plt.hlines(N - 1 - i, birthtimes[i], birthtimes[i] + lifetimes[i], color=c, lw=3)
-        m = "|" if not censorship[i] else 'o'
+        m = "|" if not event_observed[i] else 'o'
         plt.scatter((birthtimes[i]) + lifetimes[i], N - 1 - i, color=c, s=30, marker=m)
 
     plt.ylim(-0.5, N)
@@ -53,17 +54,16 @@ def shaded_plot(x, y, y_upper, y_lower, **kwargs):
 
 
 def plot_regressions(self):
-    def plot(ix=None, iloc=None, columns=[], ci_legend=False, ci_force_lines=False, **kwargs):
+    def plot(ix=None, iloc=None, columns=[], legend=True, **kwargs):
         """"
         A wrapper around plotting. Matplotlib plot arguments can be passed in, plus:
 
-          flat: a design style with stepped lines and no shading. Similar to R.
-          ci_force_lines: force the confidence intervals to be line plots (versus default shaded areas).
-          ci_legend: if ci_force_lines, boolean flag to add the line's label to the legend.
           ix: specify a time-based subsection of the curves to plot, ex:
                    .plot(ix=slice(0.,10.)) will plot the time values between t=0. and t=10.
           iloc: specify a location-based subsection of the curves to plot, ex:
                    .plot(iloc=slice(0,10)) will plot the first 10 time points.
+          columns: If not empty, plot a subset of columns from the cumulative_hazards_. Default all.
+          legend: show legend in figure.
 
         """
         assert (ix is None or iloc is None), 'Cannot set both ix and iloc in call to .plot'
@@ -88,19 +88,17 @@ def plot_regressions(self):
             y_lower = get_loc(self.confidence_intervals_[column].ix['lower']).values
             shaded_plot(x, y, y_upper, y_lower, ax=kwargs["ax"], label=column)
 
+        if legend:
+            kwargs["ax"].legend()
+
         return kwargs["ax"]
     return plot
 
-
-def coalesce(*args):
-  return next( s for s in args if s)
-
-def plot_dataframes(self, estimate):
-    def plot(ix=None, iloc=None, flat=False, show_censors=False, censor_styles={},
-             ci_legend=False, ci_force_lines=False, ci_alpha=0.3, ci_show=True,
-             bandwidth=None, **kwargs):
-        """"
-        A wrapper around plotting. Matplotlib plot arguments can be passed in, plus
+def plot_estimate(self, estimate):
+    doc_string = """"
+        Plots a pretty version of the fitted %s. 
+        
+        Matplotlib plot arguments can be passed in inside the kwargs, plus
 
         Parameters:
           flat: an opiniated design style with stepped lines and no shading. Similar to R's plotting. Default: False
@@ -116,19 +114,25 @@ def plot_dataframes(self, estimate):
                    .plot(iloc=slice(0,10)) will plot the first 10 time points.
           bandwidth: specify the bandwidth of the kernel smoother for the smoothed-hazard rate. Only used
               when called 'plot_hazard'.
+
         Returns:
           ax: a pyplot axis object
-        """
+        """%estimate
+
+    def plot(ix=None, iloc=None, flat=False, show_censors=False, censor_styles={},
+             ci_legend=False, ci_force_lines=False, ci_alpha=0.25, ci_show=True,
+             bandwidth=None, **kwargs):
+
         assert (ix is None or iloc is None), 'Cannot set both ix and iloc in call to .plot().'
 
         if "ax" not in kwargs:
             kwargs["ax"] = plt.figure().add_subplot(111)
         kwargs['color'] = coalesce( kwargs.get('c'), kwargs.get('color'), next(kwargs["ax"]._get_lines.color_cycle) )
+        kwargs['drawstyle'] = coalesce( kwargs.get('drawstyle'), 'steps-post')
 
         # R-style graphics
         if flat:
             ci_force_lines = True
-            kwargs["drawstyle"] = "steps-post"
             show_censors = True
 
         if estimate == "hazard_":
@@ -149,10 +153,10 @@ def plot_dataframes(self, estimate):
         get_loc = lambda df: getattr(df, get_method)[user_submitted_ix]
 
         # plot censors
-        if show_censors and self.event_times['censored'].sum() > 0:
-            cs = {'marker':'+'}
+        if show_censors and self.event_table['censored'].sum() > 0:
+            cs = {'marker':'+','ms':12, 'mew':1}
             cs.update(censor_styles)
-            times = get_loc(self.event_times.ix[(self.event_times['censored'] > 0)]).index.values.astype(float)
+            times = get_loc(self.event_table.ix[(self.event_table['censored'] > 0)]).index.values.astype(float)
             v = self.predict(times)
             kwargs['ax'].plot(times, v, linestyle='None', color=kwargs['color'], **cs )
                                         
@@ -162,15 +166,61 @@ def plot_dataframes(self, estimate):
         # plot confidence intervals
         if ci_show:
             if ci_force_lines:
-                get_loc(confidence_interval_).plot(linestyle="-", linewidth=1,
-                                                   c=kwargs['color'], legend=ci_legend,
+               get_loc(confidence_interval_).plot(linestyle="-", linewidth=1,
+                                                   c=kwargs['color'], legend=True,
                                                    drawstyle=kwargs.get('drawstyle', 'default'), 
-                                                   ax=kwargs['ax'])
+                                                   ax=kwargs['ax'], alpha=0.6)
             else:
                 x = get_loc(confidence_interval_).index.values.astype(float)
                 lower = get_loc(confidence_interval_.filter(like='lower')).values[:, 0]
                 upper = get_loc(confidence_interval_.filter(like='upper')).values[:, 0]
-                kwargs['ax'].fill_between(x, lower, y2=upper, alpha=ci_alpha, color=kwargs['color'])
+                fill_between_steps(x, lower, y2=upper, ax=kwargs['ax'], alpha=ci_alpha, color=kwargs['color'], linewidth=1.0)
 
         return kwargs['ax']
+    plot.__doc__ = doc_string
     return plot
+
+
+def fill_between_steps(x, y1, y2=0, h_align='left', ax=None, **kwargs):
+    ''' Fills a hole in matplotlib: Fill_between for step plots.
+    https://gist.github.com/thriveth/8352565
+ 
+    Parameters :
+    ------------
+ 
+    x : array-like
+        Array/vector of index values. These are assumed to be equally-spaced.
+        If not, the result will probably look weird...
+    y1 : array-like
+        Array/vector of values to be filled under.
+    y2 : array-Like
+        Array/vector or bottom values for filled area. Default is 0.
+ 
+    **kwargs will be passed to the matplotlib fill_between() function.
+ 
+    '''
+    # If no Axes opject given, grab the current one:
+    if ax is None:
+        ax = plt.gca()
+    # First, duplicate the x values
+    xx = x.repeat(2)[1:]
+    # Now: the average x binwidth
+    xstep = (x[1:] - x[:-1]).mean()
+    # Now: add one step at end of row.
+    xx = np.append(xx, xx.max() + xstep)
+ 
+    # Make it possible to change step alignment.
+    if h_align == 'mid':
+        xx -= xstep / 2.
+    elif h_align == 'right':
+        xx -= xstep
+ 
+    # Also, duplicate each y coordinate in both arrays
+    y1 = y1.repeat(2)
+    if type(y2) == np.ndarray:
+        y2 = y2.repeat(2)
+ 
+    # now to the plotting part:
+    ax.fill_between(xx, y1, y2=y2, **kwargs)
+ 
+    return ax
