@@ -5,6 +5,7 @@ python -m lifelines.tests.test_suit
 from __future__ import print_function
 
 import unittest
+from StringIO import StringIO
 
 import numpy as np
 import numpy.testing as npt
@@ -12,11 +13,12 @@ from collections import Counter
 import matplotlib.pyplot as plt
 import pandas as pd
 
-from ..estimation import KaplanMeierFitter, NelsonAalenFitter, AalenAdditiveFitter, median_survival_times
+from ..estimation import KaplanMeierFitter, NelsonAalenFitter, AalenAdditiveFitter, \
+                         median_survival_times, BreslowFlemingHarringtonFitter, BayesianFitter
 from ..statistics import logrank_test, multivariate_logrank_test, pairwise_logrank_test
 from ..generate_datasets import *
 from ..plotting import plot_lifetimes
-from ..utils import datetimes_to_durations, survival_events_from_table, survival_table_from_events
+from ..utils import datetimes_to_durations, survival_events_from_table, survival_table_from_events,StatError
 
 
 class MiscTests(unittest.TestCase):
@@ -33,19 +35,8 @@ class MiscTests(unittest.TestCase):
         x = np.linspace(0, 4, 1000)[:, None]
         answer = np.exp(x) - 1.
         approx_answer = cumulative_quadrature(np.exp(x).T, x).T
-        # pdb.set_trace()
         npt.assert_almost_equal(answer, approx_answer, decimal=4)
 
-    def test_aalen_additive_allows_numpy_or_df(self):
-        t = np.random.random((10, 1))
-        dfX = pd.DataFrame(np.random.random((10, 3)), columns=["A", "B", "C"])
-        c = dfX.columns
-        npX = np.random.random((10, 5))
-        aaf = AalenAdditiveFitter()
-        aaf.fit(t, npX)
-        aaf.fit(t, dfX)
-        npt.assert_array_equal(c, aaf.cumulative_hazards_.columns[:3])
-
     def test_datetimes_to_durations_days(self):
         start_date = ['2013-10-10 0:00:00', '2013-10-09', '2012-10-10']
         end_date = ['2013-10-13', '2013-10-10 0:00:00', '2013-10-15']
@@ -95,8 +86,8 @@ class MiscTests(unittest.TestCase):
 
     def test_survival_table_to_events(self):
         T, C = np.array([1, 2, 3, 4, 4, 5]), np.array([1, 0, 1, 1, 1, 1])
-        d = survival_table_from_events(T, C)
-        T_, C_ = survival_events_from_table(d)
+        d = survival_table_from_events(T, C, np.zeros_like(T))
+        T_, C_ = survival_events_from_table(d[['censored', 'observed']])
         npt.assert_array_equal(T, T_)
         npt.assert_array_equal(C, C_)
 
@@ -122,12 +113,12 @@ class StatisticalTests(unittest.TestCase):
 
     def test_censor_nelson_aalen(self):
         naf = NelsonAalenFitter(nelson_aalen_smoothing=False)
-        naf.fit(LIFETIMES, censorship=CENSORSHIP)
+        naf.fit(LIFETIMES, event_observed=OBSERVED)
         npt.assert_almost_equal(naf.cumulative_hazard_.values, self.nac)
 
     def test_censor_kaplan_meier(self):
         kmf = KaplanMeierFitter()
-        kmf.fit(LIFETIMES, censorship=CENSORSHIP)
+        kmf.fit(LIFETIMES, event_observed=OBSERVED)
         npt.assert_almost_equal(kmf.survival_function_.values, self.kmc)
 
     def test_median(self):
@@ -159,12 +150,12 @@ class StatisticalTests(unittest.TestCase):
         print(summary)
         self.assertTrue(result)
 
-    def test_unequal_intensity_censorship(self):
+    def test_unequal_intensity_event_observed(self):
         data1 = np.random.exponential(5, size=(200, 1))
         data2 = np.random.exponential(1, size=(200, 1))
-        censorA = np.random.binomial(1, 0.5, size=(200, 1))
-        censorB = np.random.binomial(1, 0.5, size=(200, 1))
-        summary, p_value, result = logrank_test(data1, data2, censorship_A=censorA, censorship_B=censorB)
+        eventA = np.random.binomial(1, 0.5, size=(200, 1))
+        eventB = np.random.binomial(1, 0.5, size=(200, 1))
+        summary, p_value, result = logrank_test(data1, data2, event_observed_A=eventA, event_observed_B=eventB)
         print(summary)
         self.assertTrue(result)
 
@@ -198,7 +189,7 @@ class StatisticalTests(unittest.TestCase):
         naf.smoothed_hazard_(1.)
         self.assertTrue(True)
 
-    def test_smoothing_hazard_ties_no_censorship(self):
+    def test_smoothing_hazard_ties_no_event_observed(self):
         T = np.random.binomial(20, 0.7, size=300)
         naf = NelsonAalenFitter()
         naf.fit(T)
@@ -249,39 +240,39 @@ class StatisticalTests(unittest.TestCase):
         timeline = np.linspace(0, 70, 5000)
         hz, coef, X = generate_hazard_rates(n, d, timeline)
         T = generate_random_lifetimes(hz, timeline)
+        X['T'] = T
+        X['E'] = 1
         # fit it to Aalen's model
         aaf = AalenAdditiveFitter(penalizer=0., fit_intercept=False)
-        aaf.fit(T, X, censorship=None)
+        aaf.fit(X)
 
         # predictions
-        T_pred = aaf.predict_median(X)
+        T_pred = aaf.predict_median(X[range(6)])
         self.assertTrue(abs((T_pred.values > T).mean() - 0.5) < 0.05)
 
+
     def test_aalen_additive_fit_no_censor(self):
         # this is a visual test of the fitting the cumulative
         # hazards.
-        n = 2500
-        d = 3
+        n = 250
+        d = 2
         timeline = np.linspace(0, 70, 5000)
         hz, coef, X = generate_hazard_rates(n, d, timeline)
-        cumulative_hazards = cumulative_quadrature(coef.values.T, timeline).T
+        X.columns = coef.columns
+        cumulative_hazards = pd.DataFrame(cumulative_quadrature(coef.values.T, timeline).T, 
+                                          index=timeline, columns=coef.columns)
         T = generate_random_lifetimes(hz, timeline)
+        X['T'] = T
+        X['E'] = 1
+        aaf = AalenAdditiveFitter(penalizer=1., fit_intercept=False)
+        aaf.fit(X)
 
-        # fit the aaf, no intercept as it is already built into X, X[2] is ones
-        aaf = AalenAdditiveFitter(penalizer=0., fit_intercept=False)
-        aaf.fit(T, X, censorship=None, columns=coef.columns)
 
-        T_max = aaf.timeline[-10]
-        # plot baby
-        for i, column in enumerate(coef.columns):
-            ax = plt.subplot(d + 2, 1, i + 1)
-            ax.plot(timeline[timeline < T_max], cumulative_hazards[timeline < T_max, i])
-            aaf.cumulative_hazards_[column].plot(ax=ax)
-            ax.legend(loc='lower right')
-
-        ax = plt.subplot(d + 2, 1, d + 2)
-        ax.plot((T > np.arange(T_max)).sum(0), c="k", label="number of observations")
-        ax.set_xlabel("time")
+        for i in range(d+1):
+            ax = plt.subplot(d+1,1,i+1)
+            col = cumulative_hazards.columns[i]
+            ax = cumulative_hazards[col].ix[:15].plot(legend=False,ax=ax)
+            ax = aaf.plot(ix=slice(0,15),ax=ax, columns=[col], legend=False)
         plt.show()
         return
 
@@ -289,32 +280,27 @@ class StatisticalTests(unittest.TestCase):
         # this is a visual test of the fitting the cumulative
         # hazards.
         n = 2500
-        d = 3
+        d = 5
         timeline = np.linspace(0, 70, 5000)
         hz, coef, X = generate_hazard_rates(n, d, timeline)
-        cumulative_hazards = cumulative_quadrature(coef.values.T, timeline).T
+        X.columns = coef.columns
+        cumulative_hazards = pd.DataFrame(cumulative_quadrature(coef.values.T, timeline).T, 
+                                          index=timeline, columns=coef.columns)
         T = generate_random_lifetimes(hz, timeline)
-        C = np.random.binomial(1, 0.8, size=n)
+        X['T'] = T
+        X['E'] = np.random.binomial(1,0.99,n)
+        aaf = AalenAdditiveFitter(penalizer=1., fit_intercept=False)
+        aaf.fit(X)
 
-        # fit the aaf, no intercept as it is already built into X, X[2] is ones
-        aaf = AalenAdditiveFitter(penalizer=0., fit_intercept=False)
-        aaf.fit(T, X, censorship=C, columns=coef.columns)
 
-        T_max = aaf.timeline[-10]
-        # plot baby
-        for i, column in enumerate(coef.columns):
-            ax = plt.subplot(d + 2, 1, i + 1)
-            ax.plot(timeline[timeline < T_max], cumulative_hazards[timeline < T_max, i])
-            aaf.plot(ax=ax, columns=[column], iloc=slice(0, 200))
-            ax.legend(loc='lower right')
-
-        ax = plt.subplot(d + 2, 1, d + 2)
-        ax.plot((T > np.arange(T_max)).sum(0), c="k", label="number of observations")
-        ax.set_xlabel("time")
-        plt.suptitle("aalen fit with 0.8 observations, d = %d" % d)
+        for i in range(d+1):
+            ax = plt.subplot(d+1,1,i+1)
+            col = cumulative_hazards.columns[i]
+            ax = cumulative_hazards[col].ix[:15].plot(legend=False,ax=ax)
+            ax = aaf.plot(ix=slice(0,15),ax=ax, columns=[col], legend=False)
         plt.show()
-        return
-
+        return  
+    
     def test_lists_to_KaplanMeierFitter(self):
         T = [2, 3, 4., 1., 6, 5.]
         C = [1, 0, 0, 0, 1, 1]
@@ -347,7 +333,7 @@ class StatisticalTests(unittest.TestCase):
         df["C"] = np.random.binomial(1, 0.6, size=N)
         df["group"] = np.random.binomial(2, 0.5, size=N)
         try:
-            pairwise_logrank_test(df['T'], df["group"], censorship=df["C"])
+            pairwise_logrank_test(df['T'], df["group"], event_observed=df["C"])
             self.assertTrue(True)
         except:
             self.assertTrue(False)
@@ -364,38 +350,75 @@ class StatisticalTests(unittest.TestCase):
         naf.fit(T, C).plot()
         plt.title("Should be a linear with slope = 0.1")
 
-    def test_kmf_naf_insert_0(self):
-        kmf, naf = KaplanMeierFitter(), NelsonAalenFitter()
-        T = [1, 2, 3, 4]
-        kmf.fit(T), naf.fit(T)
-        self.assertTrue(int(kmf.survival_function_.index[0]) == 0)
-        self.assertTrue(int(naf.cumulative_hazard_.index[0]) == 0)
-
-        kmf.fit(T, insert_0=False), naf.fit(T, insert_0=False)
-        self.assertTrue(int(kmf.survival_function_.index[0]) == 1)
-        self.assertTrue(int(naf.cumulative_hazard_.index[0]) == 1)
-
     def test_subtraction_function(self):
-
         kmf = KaplanMeierFitter()
         kmf.fit(waltonT)
         npt.assert_array_almost_equal(kmf.subtract(kmf).sum().values, 0.0)
 
     def test_divide_function(self):
-
         kmf = KaplanMeierFitter()
         kmf.fit(waltonT)
         npt.assert_array_almost_equal(np.log(kmf.divide(kmf)).sum().values, 0.0)
 
+    def test_kmf_minimum_observation_bias(self):
+        N = 250
+        kmf = KaplanMeierFitter()
+        T, C = exponential_survival_data(N, 0.2, scale=10)
+        B, _ = exponential_survival_data(N, 0.0, scale=2)
+        kmf.fit(T,C, entry=B)
+        kmf.plot()
+        plt.title("Should have larger variances in the tails")
+
+    def test_stat_error(self):
+        births = np.array([51, 58, 55, 28, 21, 19, 25, 48, 47, 25, 31, 24, 25, 30, 33, 36, 30, \
+            41, 43, 45, 35, 29, 35, 32, 36, 32, 10])
+        observations = np.array([ 1,  1,  2, 22, 30, 28, 32, 11, 14, 36, 31, 33, 33, 37, 35, 25, 31, \
+            22, 26, 24, 35, 34, 30, 35, 40, 39,  2])
+        kmf = KaplanMeierFitter()
+        with self.assertRaises(StatError):
+            kmf.fit(observations, entry=births)
+
+    def test_BHF_fit(self):
+        bfh = BreslowFlemingHarringtonFitter()
+        births = np.array([51, 58, 55, 28, 21, 19, 25, 48, 47, 25, 31, 24, 25, 30, 33, 36, 30, \
+           41, 43, 45, 35, 29, 35, 32, 36, 32, 10])
+        observations = np.array([ 1,  1,  2, 22, 30, 28, 32, 11, 14, 36, 31, 33, 33, 37, 35, 25, 31, \
+           22, 26, 24, 35, 34, 30, 35, 40, 39,  2])
+        bfh.fit(observations, entry=births)
+        return 
+
+    def test_aaf_panel_dataset(self):
+        aaf = AalenAdditiveFitter()
+        aaf.fit(panel_dataset, id_col='id',duration_col='t', event_col='E')
+        aaf.plot()
+        return
+
+    def test_bayesian_fitter_low_data(self):
+        bf = BayesianFitter(samples=10)
+        bf.fit(waltonT1)
+        ax = bf.plot()
+
+        bf.fit(waltonT2)
+        bf.plot(ax=ax,c='#A60628')
+        return
+
+    def test_bayesian_fitter_large_data(self):
+        bf = BayesianFitter()
+        bf.fit(np.random.exponential(10,size=1000))
+        bf.plot()
+        return
+
+
     def kaplan_meier(self, censor=False):
         km = np.zeros((len(self.lifetimes.keys()), 1))
         ordered_lifetimes = np.sort(self.lifetimes.keys())
+        N = len(LIFETIMES)
         v = 1.
         n = N * 1.0
         for i, t in enumerate(ordered_lifetimes):
             if censor:
                 ix = LIFETIMES == t
-                c = sum(1 - CENSORSHIP[ix])
+                c = sum(1 - OBSERVED[ix])
                 if n != 0:
                     v *= (1 - (self.lifetimes.get(t) - c) / n)
                 n -= self.lifetimes.get(t)
@@ -410,12 +433,13 @@ class StatisticalTests(unittest.TestCase):
     def nelson_aalen(self, censor=False):
         na = np.zeros((len(self.lifetimes.keys()), 1))
         ordered_lifetimes = np.sort(self.lifetimes.keys())
+        N = len(LIFETIMES)
         v = 0.
         n = N * 1.0
         for i, t in enumerate(ordered_lifetimes):
             if censor:
                 ix = LIFETIMES == t
-                c = sum(1 - CENSORSHIP[ix])
+                c = sum(1 - OBSERVED[ix])
                 if n != 0:
                     v += ((self.lifetimes.get(t) - c) / n)
                 n -= self.lifetimes.get(t)
@@ -435,15 +459,18 @@ class PlottingTests(unittest.TestCase):
         # hazards.
         n = 2500
         d = 3
-        timeline = np.linspace(0, 70, 5000)
+        timeline = np.linspace(0, 70, 10000)
         hz, coef, X = generate_hazard_rates(n, d, timeline)
         cumulative_hazards = cumulative_quadrature(coef.values.T, timeline).T
         T = generate_random_lifetimes(hz, timeline)
-        C = np.random.binomial(1, 0.8, size=n)
+        C = np.random.binomial(1, 1., size=n)
+        X['T'] = T
+        X['E'] = C
 
         # fit the aaf, no intercept as it is already built into X, X[2] is ones
         aaf = AalenAdditiveFitter(penalizer=0., fit_intercept=False)
-        aaf.fit(T, X, censorship=C, columns=coef.columns)
+
+        aaf.fit(X)
         ax = aaf.plot(iloc=slice(0, aaf.cumulative_hazards_.shape[0] - 100))
         ax.set_xlabel("time")
         ax.set_title('.plot() cumulative hazards')
@@ -459,10 +486,12 @@ class PlottingTests(unittest.TestCase):
         cumulative_hazards = cumulative_quadrature(coef.values.T, timeline).T
         T = generate_random_lifetimes(hz, timeline) + 0.1 * np.random.uniform(size=(n, 1))
         C = np.random.binomial(1, 0.8, size=n)
+        X['T'] = T
+        X['E'] = C
 
         # fit the aaf, no intercept as it is already built into X, X[2] is ones
         aaf = AalenAdditiveFitter(penalizer=0., fit_intercept=False)
-        aaf.fit(T, X, censorship=C, columns=coef.columns)
+        aaf.fit(X)
         ax = aaf.smoothed_hazards_(1).iloc[0:aaf.cumulative_hazards_.shape[0] - 500].plot()
         ax.set_xlabel("time")
         ax.set_title('.plot() smoothed hazards')
@@ -529,7 +558,7 @@ class PlottingTests(unittest.TestCase):
         current = 10
         birthtimes = current * np.random.uniform(size=(N,))
         T, C = generate_random_lifetimes(hz, t, size=N, censor=current - birthtimes)
-        plot_lifetimes(T, censorship=C, birthtimes=birthtimes)
+        plot_lifetimes(T, event_observed=C, birthtimes=birthtimes)
 
     def test_plot_lifetimes_relative(self):
         plt.figure()
@@ -537,7 +566,7 @@ class PlottingTests(unittest.TestCase):
         hz, coef, covrt = generate_hazard_rates(1, 5, t)
         N = 20
         T, C = generate_random_lifetimes(hz, t, size=N, censor=True)
-        plot_lifetimes(T, censorship=C)
+        plot_lifetimes(T, event_observed=C)
 
     def test_naf_plot_cumulative_hazard(self):
         data1 = np.random.exponential(5, size=(200, 1))
@@ -602,11 +631,11 @@ class PlottingTests(unittest.TestCase):
 
 # some data
 LIFETIMES = np.array([2, 4, 4, 4, 5, 7, 10, 11, 11, 12])
-CENSORSHIP = np.array([1, 1, 0, 1, 0, 1, 1, 1, 1, 0])
+OBSERVED = np.array([1, 1, 0, 1, 0, 1, 1, 1, 1, 0])
 N = len(LIFETIMES)
 
-waltonT = np.array([6.,13.,13.,13.,19.,19.,19.,26.,26.,26.,26.,26.,33.,33.,47.,62.,62.,9.,9.,9.,15.,15.,22.,22.,22.,22.,29.,29.,29.,29.,29.,36.,36.,43.])
-waltonT = np.array([33.,54.,54.,61.,61.,61.,61.,61.,61.,61.,61.,61.,61.,61.,69.,69.,69.,69.,69.,69.,69.,69.,69.,69.,69.,32.,53.,53.,60.,60.,60.,60.,60.,
+waltonT1 = np.array([6.,13.,13.,13.,19.,19.,19.,26.,26.,26.,26.,26.,33.,33.,47.,62.,62.,9.,9.,9.,15.,15.,22.,22.,22.,22.,29.,29.,29.,29.,29.,36.,36.,43.])
+waltonT2 = np.array([33.,54.,54.,61.,61.,61.,61.,61.,61.,61.,61.,61.,61.,61.,69.,69.,69.,69.,69.,69.,69.,69.,69.,69.,69.,32.,53.,53.,60.,60.,60.,60.,60.,
                         68.,68.,68.,68.,68.,68.,68.,68.,68.,68.,75.,17.,51.,51.,51.,58.,58.,58.,58.,66.,66.,7.,7.,41.,41.,41.,41.,41.,41.,41.,48.,48.,48.,
                         48.,48.,48.,48.,48.,56.,56.,56.,56.,56.,56.,56.,56.,56.,56.,56.,56.,56.,56.,56.,56.,56.,56.,63.,63.,63.,63.,63.,63.,63.,63.,63.,69.,
                         69.,38.,38.,45.,45.,45.,45.,45.,45.,45.,45.,45.,45.,53.,53.,53.,53.,53.,60.,60.,60.,60.,60.,60.,60.,60.,60.,60.,60.,66.])
@@ -657,5 +686,39 @@ waltonT = np.array([6., 13., 13., 13., 19., 19., 19., 26., 26., 26., 26.,
                     60., 60., 60., 60., 60., 60., 60., 60., 66.])
 
 
+panel_dataset = pd.read_csv(
+    StringIO("""id,t,E,var1,var2
+1,1,0,0,1
+1,2,0,0,1
+1,3,0,4,3
+1,4,1,8,4
+2,1,0,1.2,1
+2,2,0,1.2,2
+2,3,0,1.2,2
+3,1,0,0,1
+3,2,1,1,2
+4,1,0,0,1
+4,2,0,1,2
+4,3,0,1,3
+4,4,0,2,4
+4,5,1,2,5
+5,1,0,1,-1
+5,2,0,2,-1
+5,3,0,3,-1
+6,1,1,3,0
+7,1,0,1,0
+7,2,0,2,1
+7,3,0,3,0
+7,4,0,3,1
+7,5,0,3,0
+7,6,1,3,1
+8,1,0,-1,0
+8,2,1,1,0
+9,1,0,1,1
+9,2,0,2,2
+"""))
+
+
+
 if __name__ == '__main__':
     unittest.main()
