@@ -6,16 +6,21 @@ import numpy as np
 import pandas as pd
 from pandas import to_datetime
 
+def coalesce(*args):
+  return next( s for s in args if s)
 
-def group_survival_table_from_events(groups, durations, censorship, limit=-1):
+def group_survival_table_from_events(groups, durations, event_observed, min_observations, limit=-1):
     """
     Joins multiple event series together into dataframes. A generalization of
     `survival_table_from_events` to data with groups. Previously called `group_event_series` pre 0.2.3.
 
     Parameters:
-        groups: a (n,) array of individuals' group ids.
-        durations: a (n,) array of durations of each individual
-        censorship: a (n,) array of censorship, 1 if observed, 0 else.
+        groups: a (n,) numpy array of individuals' group ids.
+        durations: a (n,) numpy array of durations of each individual
+        event_observed: a (n,) numpy array of event observations, 1 if observed, 0 else.
+        event_observed: a (n,) numpy array of times individual entered study. This is most applicable in 
+                    cases where there is left-truncation, i.e. a individual might enter the 
+                    study late. If not the case, normally set to all zeros. 
 
     Output:
         - np.array of unique groups
@@ -38,7 +43,7 @@ def group_survival_table_from_events(groups, durations, censorship, limit=-1):
             9                       0                3
             13                      0                3
             15                      0                2
-            ...,
+            ,
                       observed:control  observed:miR-137
             event_at
             6                        0                 1
@@ -46,13 +51,13 @@ def group_survival_table_from_events(groups, durations, censorship, limit=-1):
             9                        0                 3
             13                       0                 3
             15                       0                 2
-            ...,
+            ,
                       censored:control  censored:miR-137
             event_at
             6                        0                 0
             7                        0                 0
             9                        0                 0
-            ...,
+            ,
         ]
 
     """
@@ -62,28 +67,37 @@ def group_survival_table_from_events(groups, durations, censorship, limit=-1):
     g = unique_groups[0]
     ix = groups == g
     T = durations[ix]
-    C = censorship[ix]
+    C = event_observed[ix]
+    B = min_observations[ix]
 
     g_name = str(g)
-    data = survival_table_from_events(T, C, columns=['removed:' + g_name, "observed:" + g_name, 'censored:' + g_name])
+    data = survival_table_from_events(T, C, B, 
+                columns=['removed:' + g_name, "observed:" + g_name, 'censored:' + g_name, 'entrance' + g_name])
     for g in unique_groups[1:]:
         ix = groups == g
         T = durations[ix]
-        C = censorship[ix]
+        C = event_observed[ix]
+        B = min_observations[ix]
         g_name = str(g)
-        data = data.join(survival_table_from_events(T, C, columns=['removed:' + g_name, "observed:" + g_name, 'censored:' + g_name]), how='outer')
+        data = data.join(survival_table_from_events(T, C, B, 
+                    columns=['removed:' + g_name, "observed:" + g_name, 'censored:' + g_name, 'entrance' + g_name]),
+                    how='outer')
     data = data.fillna(0)
-    # hmmm pandas...its too bad I can't do data.ix[:limit] and leave out the if.
+    # hmmm pandasits too bad I can't do data.ix[:limit] and leave out the if.
     if int(limit) != -1:
         data = data.ix[:limit]
     return unique_groups, data.filter(like='removed:'), data.filter(like='observed:'), data.filter(like='censored:')
 
 
-def survival_table_from_events(event_times, censorship, columns=["removed", "observed", "censored"], weights=None):
+def survival_table_from_events(durations, event_observed, min_observations,
+                              columns=["removed", "observed", "censored", 'entrance'], weights=None):
     """
     Parameters:
-        event_times: (n,1) array of event times
-        censorship: if not None, (n,1) boolean array, 1 if observed event, 0 is censored
+        durations: (n,1) array of event times (durations individual was observed for)
+        event_observed: (n,1) boolean array, 1 if observed event, 0 is censored event.
+        min_observations: used for left truncation data. Sometimes subjects will show 
+          up late in the study. min_observations is a (n,1) array of positive numbers representing
+          when the subject was first observed. A subject's life is then [min observation + duration observed]
         columns: a 3-length array to call the, in order, removed individuals, observed deaths
           and censorships.
 
@@ -93,7 +107,7 @@ def survival_table_from_events(event_times, censorship, columns=["removed", "obs
         by the end of the period. The column 'observed' refers to the number of removed
         individuals who were observed to have died (i.e. not censored.) The column
         'censored' is defined as 'removed' - 'observed' (the number of individuals who
-         left the population due to censorship)
+         left the population due to event_observed)
 
     Example:
         #input
@@ -110,12 +124,23 @@ def survival_table_from_events(event_times, censorship, columns=["removed", "obs
         15              2         2         0
 
     """
-    event_times = np.array(event_times)
-    df = pd.DataFrame(event_times.astype(float), columns=["event_at"])
+    #deal with deaths and censorships
+
+    durations = np.asarray(durations) + min_observations
+    df = pd.DataFrame(durations, columns=["event_at"])
     df[columns[0]] = 1 if weights is None else weights
-    df[columns[1]] = censorship
-    event_table = df.groupby("event_at").sum().sort_index()
-    event_table[columns[2]] = event_table[columns[0]] - event_table[columns[1]]
+    df[columns[1]] = event_observed
+    death_table = df.groupby("event_at").sum()
+    death_table[columns[2]] = (death_table[columns[0]] - death_table[columns[1]]).astype(int)
+
+    #deal with late births
+    births = pd.DataFrame( min_observations, columns=['event_at'])
+    births[columns[3]] = 1
+    births_table = births.groupby('event_at').sum()
+ 
+    #this next line can be optimized for when min_observerations is all zeros.     
+    event_table = death_table.join(births_table, how='outer', sort=True).fillna(0) #http://wesmckinney.com/blog/?p=414
+
     return event_table
 
 
@@ -129,7 +154,7 @@ def survival_events_from_table(event_table, observed_deaths_col="observed", cens
 
     Returns
         T: a np.array of durations of observation -- one element for each individual in the population.
-        C: a np.array of censorships -- one element for each individual in the population. 1 if observed, 0 else.
+        C: a np.array of event observations -- one element for each individual in the population. 1 if observed, 0 else.
 
     Ex: The survival table, as a pandas DataFrame:
 
@@ -163,7 +188,7 @@ def survival_events_from_table(event_table, observed_deaths_col="observed", cens
 def datetimes_to_durations(start_times, end_times, fill_date=datetime.today(), freq='D', dayfirst=False, na_values=None):
     """
     This is a very flexible function for transforming arrays of start_times and end_times
-    to the proper format for lifelines: duration and censorship arrays.
+    to the proper format for lifelines: duration and event observation arrays.
 
     Parameters:
         start_times: an array, series or dataframe of start times. These can be strings, or datetimes.
@@ -177,7 +202,7 @@ def datetimes_to_durations(start_times, end_times, fill_date=datetime.today(), f
 
     Returns:
         T: a array of floats representing the durations with time units given by freq.
-        C: a boolean array of censorship: 1 if death observed, 0 else.
+        C: a boolean array of event observations: 1 if death observed, 0 else.
 
     """
     freq_string = 'timedelta64[%s]' % freq
@@ -200,6 +225,13 @@ def datetimes_to_durations(start_times, end_times, fill_date=datetime.today(), f
     return T.values, C.values
 
 
+class StatError(Exception):
+     def __init__(self, msg):
+         self.msg = msg
+     def __str__(self):
+         return repr(self.msg)
+
+
 def inv_normal_cdf(p):
     if p < 0.5:
         return -AandS_approximation(p)
@@ -227,7 +259,7 @@ def median_loss(T, T_pred):
     return np.abs(T - T_pred).mean()
 
 
-def cross_validation(fitter, T, X, censorship=None, k=5, loss_function="median"):
+def cross_validation(fitter, T, X, event_observed=None, k=5, loss_function="median"):
     """
     This implements the censor-sensative loss function as given in
     'Prediction Performance of Survival Models', Yan Yuan, 2008
@@ -235,19 +267,19 @@ def cross_validation(fitter, T, X, censorship=None, k=5, loss_function="median")
     pass
 
 
-def yaun_loss(fitter, T_true, T_pred, X_train, T_train, censorship=None):
+def yaun_loss(fitter, T_true, T_pred, X_train, T_train, event_observed=None):
     "[WIP]"
-    if censorship is None:
+    if event_observed is None:
         t = T_pred.shape[0]
         sC = np.ones((t, 1))
-        censorship = np.ones((t, 1))
+        event_observed = np.ones((t, 1))
     else:
         # We are estimating S_c(*|z),is the probability that the
         # jth individual survived to time yj without being censored
-        fitter.fit(T_train, X_train, censorship=1 - censorship, timeline=T_true)
+        fitter.fit(T_train, X_train, event_observed=1 - event_observed, timeline=T_true)
         sC = fitter.predict_survival_function(X_train).ix[T_true[:, 0]].values.diagonal()[:, None]  # dirty way to get this
 
-    return (censorship * (T_pred - T_true) ** 2 / sC).mean()
+    return (event_observed * (T_pred - T_true) ** 2 / sC).mean()
 
 
 def quadrature(fx, x):
@@ -255,12 +287,6 @@ def quadrature(fx, x):
     return (0.5 * fx[:, 0] + fx[:, 1:-1].sum(1) + 0.5 * fx[:, -1]) * (x[-1] - x[0]) * 1.0 / t
 
 
-def basis(n, i):
-    x = np.zeros((n, 1))
-    x[i] = 1
-    return x
-
-
 def epanechnikov_kernel(t, T, bandwidth=1.):
     M = 0.75 * (1 - (t - T) / bandwidth) ** 2
     M[abs((t - T)) >= bandwidth] = 0
